/*
 * MainFrame.java
 *
 * Created on May 30, 2010, 10:03:33 PM
 */

package swingworkerdemo.gui;

import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import javax.swing.SwingWorker;
import javax.swing.event.*;
import swingworkerdemo.fp.Calculation;

/**
 * The main window from which a Fibonacci calculation can be started.
 * The user can select whether this calculation runs in the GUI thread,
 * or in a background thread via a SwingWorker.
 * <p>
 * It demonstrates
 * <ul>
 * <li>how to set up the SwingWorker,
 * <li>start the calculation,
 * <li>allow it to be interrupted,
 * <li>let it report the final result.
 * </ul>
 *
 * @author Tom Verhoeff (TU/e)
 */
public class MainFrame extends javax.swing.JFrame {

    /**
     * Listener that responds to edit events in the argument text field.
     */
    class ArgumentListener implements DocumentListener {
        @Override
        public void changedUpdate(DocumentEvent e) { validateArgument(); }
        @Override
        public void insertUpdate(DocumentEvent e) { validateArgument(); }
        @Override
        public void removeUpdate(DocumentEvent e) { validateArgument(); }
    }

    /** Creates new form MainFrame. */
    public MainFrame() {
        initComponents(); // generated by NetBeans
        initFrame();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jTextFieldArgument = new javax.swing.JTextField();
        jTextFieldResult = new javax.swing.JTextField();
        jButtonStart = new javax.swing.JButton();
        jButtonAbort = new javax.swing.JButton();
        jCheckBoxUseSwingWorker = new javax.swing.JCheckBox();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Demo for SwingWorker to Run Calculations Separate from GUI");

        jTextFieldArgument.setColumns(6);
        jTextFieldArgument.setText("argument");
        jTextFieldArgument.setToolTipText("argument");

        jTextFieldResult.setColumns(20);
        jTextFieldResult.setEditable(false);
        jTextFieldResult.setText("result");
        jTextFieldResult.setToolTipText("result");

        jButtonStart.setText("Fibonacci");
        jButtonStart.setEnabled(false);
        jButtonStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonStartActionPerformed(evt);
            }
        });

        jButtonAbort.setText("Abort");
        jButtonAbort.setEnabled(false);
        jButtonAbort.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonAbortActionPerformed(evt);
            }
        });

        jCheckBoxUseSwingWorker.setLabel("Do calculation in a separate thread using a SwingWorker");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(jCheckBoxUseSwingWorker)
                        .addContainerGap())
                    .add(layout.createSequentialGroup()
                        .add(jTextFieldArgument, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jButtonAbort, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 104, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jButtonStart))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jTextFieldResult, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 223, Short.MAX_VALUE)
                        .add(29, 29, 29))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(17, 17, 17)
                .add(jCheckBoxUseSwingWorker)
                .add(18, 18, 18)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jTextFieldArgument, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jTextFieldResult, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jButtonStart))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jButtonAbort)
                .addContainerGap(55, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jButtonStartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonStartActionPerformed
        // this button is enabled if and only if argument is valid
        jButtonStart.setEnabled(false); // do not allow multiple invocations
        jTextFieldResult.setText("... working ..."); // show we're working

        if (jCheckBoxUseSwingWorker.isSelected()) { // execute in separate thread
            // Instances of javax.swing.SwingWorker are not reusuable, so
            // need to create a new FibonacciWorker for each invocation.
            fibonacciWorker = new FibonacciWorker();
            fibonacciWorker.execute(); // start the calculation
            // Calculation result is handled by fibonacciWorker.done().
            // GUI thread continues here.
            jButtonAbort.setEnabled(true); // allow abort
        }
        else { // execute directly in GUI thread
            final int argument = Integer.parseInt(jTextFieldArgument.getText());
            final long result = calculation.fib(argument); // may take time
            presentResult(result);
            jButtonStart.setEnabled(true);
        }
    }//GEN-LAST:event_jButtonStartActionPerformed

    private void jButtonAbortActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonAbortActionPerformed
        if (fibonacciWorker != null) {
            // If it would not modify global data structures,
            // then this could also be accomplies by.
            // fibonacciWorker.cancel(true);
            calculation.abort();
        }
    }//GEN-LAST:event_jButtonAbortActionPerformed

    /**
    * @param args the command line arguments
    */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                new MainFrame().setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButtonAbort;
    private javax.swing.JButton jButtonStart;
    private javax.swing.JCheckBox jCheckBoxUseSwingWorker;
    private javax.swing.JTextField jTextFieldArgument;
    private javax.swing.JTextField jTextFieldResult;
    // End of variables declaration//GEN-END:variables

    /** The handle for the separate thread */
    private FibonacciWorker fibonacciWorker;

    /** The calculation class */
    private Calculation calculation = new Calculation();

    /**
     * Complete the initialization of this frame.
     */
    private void initFrame() {
        // set up listeners for argument text field
        jTextFieldArgument.getDocument().addDocumentListener(
                new ArgumentListener());
        jTextFieldArgument.requestFocusInWindow();
    }

    /**
     * Returns whether the text in {@code argument} is a valid
     * argument for Fibonacci.
     *
     * @return whether {@code argument} is valid
     */
    private boolean validArgument() {
        try {
            int value = Integer.parseInt(jTextFieldArgument.getText());
            // valid number
            return Calculation.validArgument(value);
        } catch (NumberFormatException e) {
            return false;
        }
    }

    /**
     * Validates the argument and, if valid, enables the
     * Fibonacci button.
     */
    private void validateArgument() {
        final boolean valid = validArgument();
        jButtonStart.setEnabled(valid);
        if (valid) {
            jTextFieldResult.setText("result not yet calculated");
        } else {
            jTextFieldResult.setText(Calculation.LOWER_BOUND +
                    " <= argument <= " + Calculation.UPPER_BOUND);
        }
    }

    /**
     * Presents result in the result text field.
     *
     * @param result  the result to present
     */
    private void presentResult(final long result) {
        jTextFieldResult.setText(result + "");
    }

    /**
     * Inner class to control separate thread for calculating
     * Fibonacci numbers.
     */
    private class FibonacciWorker extends SwingWorker<Long, Void> {

        /**
         * Execution of the Fibonacci function is done
         * in a background thread.
         * This method is called in the background thread,
         * via {@code fibonacciWorker.execute()}.
         */
        @Override
        protected Long doInBackground() throws Exception {
            return calculation.fib(
                    Integer.parseInt(jTextFieldArgument.getText()));
        }

        /**
         * After the function is evaluated it can be shown in the GUI.
         * This method is called in the GUI thread.
         */
        @Override
        protected void done() {
            jButtonAbort.setEnabled(false); // disable further aborts
            // get the result and show it to the user
            try { // get() may throw an exception
                final Long result = get();
                if (result < 0) {
                    jTextFieldResult.setText("Aborted by user");
                } else {
                    presentResult(result);
                }
            } catch (ExecutionException e) {
                jTextFieldResult.setText("Worker threw exception (see output)");
                System.out.println(e.toString()); // show details on stdout
            } catch (CancellationException e) {
                // if fibonacciWorker.cancel() was called
                jTextFieldResult.setText("Canceled");
                // System.out.println(e.toString()); // show details on stdout
            } catch (Exception e) {
                jTextFieldResult.setText("Unexpected exception (see output)");
                System.out.println(e.toString()); // show details on stdout
            }
            jButtonStart.setEnabled(true); // allow new calculation
        }

    }

}
